/*
 * Ctunnel - Cyptographic tunnel.
 * Copyright (C) 2008-2010 Jess Mahan <ctunnel@nardcore.org>
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <time.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/select.h>
#include <sys/wait.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <netinet/in_systm.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/ioctl.h>

#define KEYFILE "/.passkey"
#define PACKET_SIZE  1024

#include "config.h"
#include "ctunnel.h"
#include "comp.h"
#include "crypto.h"


struct Network *udp_bind(int port)
{
    struct Network *udp = calloc(1, sizeof(struct Network));

    udp->sockfd = socket(AF_INET, SOCK_DGRAM, 0);

    udp->addr.sin_family = AF_INET;
    udp->addr.sin_addr.s_addr = INADDR_ANY;
    udp->addr.sin_port = htons(port);
    udp->len = sizeof(struct sockaddr_in);

    if (bind(udp->sockfd, (struct sockaddr *) &udp->addr, udp->len) == -1) {
	syslog(LOG_CRIT, "UDP Bind %s (port %d)", strerror(errno), port);
        fprintf(stderr, "UDP Bind %s (port %d)\n", strerror(errno), port);
	exit(1);
    }
    return udp;
}

struct Network *udp_connect(char *ip, int port)
{
    struct Network *udp = calloc(1, sizeof(struct Network));
    struct hostent *ht;
    char host[255] = "";

    if (!isdigit(ip[0])) {
        if ((ht = gethostbyname(ip))) {
            memset(host, 0x00, 255);
            snprintf(host, 32, "%s",
                     inet_ntoa(*(struct in_addr *) ht->h_addr_list[0]));
        } else {
	    syslog(LOG_CRIT, "Cannot resolv %s", ip);
	    fprintf(stderr, "Cannot resolv %s\n", ip);
	    exit(1);
        }
    } else {
        snprintf(host, 254, "%s", ip);
    }

    udp->sockfd = socket(AF_INET, SOCK_DGRAM, 0);
    udp->addr.sin_family = AF_INET;
    udp->addr.sin_addr.s_addr = inet_addr(host);
    udp->addr.sin_port = htons(port);
    udp->len = sizeof(struct sockaddr *);

    if (udp->sockfd < 0) {
	syslog(LOG_CRIT, "UDP Connect() %s", strerror(errno));
        fprintf(stderr, "UDP Connect() %s\n", strerror(errno));
        exit(1);
    }

    return udp;
}

int tcp_listen(int port)
{
    struct sockaddr_in sin;
    int sockfd;
    int x = 1;

    sockfd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
    setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, &x, sizeof(x));
    if (sockfd == -1) {
        perror("socket()");
        exit(1);
    }

    sin.sin_family = AF_INET;
    sin.sin_addr.s_addr = INADDR_ANY;
    sin.sin_port = htons(port);

    if (bind(sockfd, (struct sockaddr *) &sin, sizeof(sin)) == -1) {
        perror("bind()");
        exit(1);
    }
    if (listen(sockfd, 20) == -1) {
        perror("listen()");
        exit(1);
    }
    return sockfd;
}
int tcp_connect(char *ip, int port)
{
    int clifd, servfd;
    struct sockaddr_in address;
    struct hostent *ht;
    char host[255] = "";

    clifd = socket(AF_INET, SOCK_STREAM, 0);
    if (clifd < 0) {
        perror("socket");
	exit(1);
    }

    if (!isdigit(ip[0])) {
        ht = gethostbyname(ip);
        memset(host, 0x00, 255);
        snprintf(host, 32, "%s",
                 inet_ntoa(*(struct in_addr *) ht->h_addr_list[0]));
    } else {
        snprintf(host, 254, "%s", ip);
    }

    address.sin_family = AF_INET;
    address.sin_addr.s_addr = inet_addr(host);
    address.sin_port = htons(port);

    servfd = connect(clifd, (struct sockaddr *) &address, sizeof(address));
    if (servfd < 0) {
        perror("connect");
	exit(1);
    }

    return clifd;
}
